package com.jiayue.ssi.controller;

import cn.hutool.core.lang.Validator;
import cn.hutool.crypto.SmUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.ssi.annotation.AgainVerify;
import com.jiayue.ssi.annotation.InterfaceLimit;
import com.jiayue.ssi.annotation.OperateLog;
import com.jiayue.ssi.backenum.AuditType;
import com.jiayue.ssi.backenum.BusinessType;
import com.jiayue.ssi.config.SendMailUtil;
import com.jiayue.ssi.constant.SecretKeyConstants;
import com.jiayue.ssi.entity.SysUser;
import com.jiayue.ssi.entity.SysUserRole;
import com.jiayue.ssi.service.SysUserRoleService;
import com.jiayue.ssi.service.SysUserService;
import com.jiayue.ssi.service.impl.SysPermissionService;
import com.jiayue.ssi.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 用户信息接口
 *
 * @author xsl
 * @since 2023/03/13
 */
@RestController
@RequestMapping("/sysUserController")
@Slf4j
public class SysUserController {
    @Autowired
    SysUserService sysUserService;
    @Autowired
    SendMailUtil sendMailUtil;
    @Autowired
    SysPermissionService sysPermissionService;
    @Autowired
    SysUserRoleService sysUserRoleService;

    /**
     * 获取用户分页信息
     *
     * @return 用户信息
     */
    @GetMapping(value = "/getAll")
    @PreAuthorize("@ss.hasPermi('system:user:list')")
    public ResponseVO getAll(Integer currentPage, Integer pageSize, String username, String phonenumber,
                             String status) {
        try {
            if (StringUtils.isNotEmpty(username)) {
                if (username.length() > 20) {
                    return ResponseVO.fail("用户账号长度不能超过20个字符！");
                }
            }
            if (StringUtils.isNotEmpty(phonenumber)) {
                if (phonenumber.length() > 11) {
                    return ResponseVO.fail("手机号码长度不能超过11个字符！");
                }
            }
            QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
            if (StringUtils.isNotEmpty(username)) {
                wrapper.eq("username", username);
            }
            if (StringUtils.isNotEmpty(phonenumber)) {
                wrapper.eq("AES_DECRYPT(UNHEX(phonenumber), '"+new String(AesUtils.key)+"')", phonenumber);
            }
            if (StringUtils.isNotEmpty(status)) {
                wrapper.eq("status", status);
            }
            Page<SysUser> result = sysUserService.page(new Page<>(currentPage, pageSize), wrapper);
            List<SysUser> records = result.getRecords();
            //遍历对象数组的方法
            records.forEach(
                    record->{
                        record.setMailbox(AesUtils.decryptStr(record.getMailbox()));
                        record.setPhonenumber(AesUtils.decryptStr(record.getPhonenumber()));
                        record.setNickname(AesUtils.decryptStr(record.getNickname()));
                    }
            );
            return ResponseVO.success(result);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取所有用户异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 获取当前用户信息
     *
     * @return 用户信息
     */
    @GetMapping(value = "/getCurrentUser")
    public ResponseVO getCurrentUser() {
        try {
            SysUser sysUser = SecurityContextUtil.getSysUser();
            // 权限集合
            Set<String> permissions = sysPermissionService.getMenuPermission(sysUser.getId());
            Map<String, Object> map = new HashMap<>();
            sysUser.setMailbox(AesUtils.decryptStr(sysUser.getMailbox()));
            sysUser.setPhonenumber(AesUtils.decryptStr(sysUser.getPhonenumber()));
            sysUser.setNickname(AesUtils.decryptStr(sysUser.getNickname()));
            map.put("sysUser", sysUser);
            map.put("permissions", permissions);
            return ResponseVO.success(map);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取当前用户异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 新增用户
     */
    @PostMapping
    @OperateLog(title = "用户管理", businessType = BusinessType.INSERT, auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:user:add')")
    public ResponseVO add(@RequestBody SysUser user) {
        if (StringUtils.isEmpty(user.getUsername())) {
            return ResponseVO.fail("用户账号不能为空!");
        } else if (sysUserService.queryUserName(user.getUsername()) != null) {
            return ResponseVO.fail(user.getUsername() + "账号已存在！");
        } else if (user.getUsername().length() < 5 || user.getUsername().length() > 20) {
            return ResponseVO.fail(user.getUsername() + "用户账号长度必须介于5和20之间！");
        }

        if (StringUtils.isEmpty(user.getPhonenumber())) {
            return ResponseVO.fail("手机号码不能为空！");
        } else if (!Validator.isMobile(user.getPhonenumber())) {
            return ResponseVO.fail("请输入正确的手机号码！");
        }
        // 加密手机号
        user.setPhonenumber(AesUtils.encryptHex(user.getPhonenumber()).toUpperCase());

        if (StringUtils.isEmpty(user.getNickname())) {
            return ResponseVO.fail("姓名不能为空！");
        } else if (user.getNickname().length() > 30) {
            return ResponseVO.fail("姓名长度不能超过30个字符！");
        }
        // 加密姓名
        user.setNickname(AesUtils.encryptHex(user.getNickname()).toUpperCase());

        if (StringUtils.isEmpty(user.getMailbox())) {
            return ResponseVO.fail("邮箱不能为空!");
        } else if (!Validator.isEmail(user.getMailbox())) {
            return ResponseVO.fail("请输入正确的邮箱地址！");
        } else if (user.getMailbox().length() > 50) {
            return ResponseVO.fail("邮箱长度不能超过50个字符！");
        } else if (sysUserService.queryMailBox(user.getMailbox()) != null) {
            return ResponseVO.fail(user.getMailbox() + "邮箱已存在！");
        }
        // 加密邮箱
        user.setMailbox(AesUtils.encryptHex(user.getMailbox()).toUpperCase());

//        // 生成8位初始密码
//        String randomPwd = RandomPwd.getRandomPwd(8);
//        user.setPassword(SmUtil.sm3(randomPwd).toUpperCase());
//        user.setErrNum(0);
//        user.setLockTime(0L);

        try {
            boolean bo = sysUserService.save(user);
            if (bo) {
                return ResponseVO.success("添加用户信息成功");
            } else {
                log.error("添加用户信息失败");
                return ResponseVO.fail("添加用户信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("添加用户信息异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 更新用户
     *
     * @param user 参数
     * @return 执行结果
     */
    @PutMapping
    @OperateLog(title = "用户管理", businessType = BusinessType.UPDATE, auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:user:edit')")
    public ResponseVO update(@RequestBody SysUser user) {
        SysUser existUser = sysUserService.getById(user.getId());
        if (existUser == null) {
            return ResponseVO.fail("非法访问不能修改!");
        }

        if (StringUtils.isEmpty(user.getUsername())) {
            return ResponseVO.fail("用户账号不能为空!");
        } else if (user.getUsername().length() < 5 || user.getUsername().length() > 20) {
            return ResponseVO.fail(user.getUsername() + "用户账号长度必须介于5和20之间！");
        } else if (!existUser.getUsername().equals(user.getUsername())) {
            if (sysUserService.queryUserName(user.getUsername()) != null) {
                return ResponseVO.fail(user.getUsername() + "账号已存在！");
            }
        }

        if (StringUtils.isEmpty(user.getMailbox())) {
            return ResponseVO.fail("邮箱不能为空!");
        } else if (!Validator.isEmail(user.getMailbox())) {
            return ResponseVO.fail("请输入正确的邮箱地址！");
        } else if (user.getMailbox().length() > 50) {
            return ResponseVO.fail("邮箱长度不能超过50个字符！");
        } else if (!existUser.getMailbox().equals(user.getMailbox())) {
            if (sysUserService.queryMailBox(user.getMailbox()) != null) {
                return ResponseVO.fail(user.getMailbox() + "邮箱已存在！");
            }
        }
        // 加密邮箱
        user.setMailbox(AesUtils.encryptHex(user.getMailbox()).toUpperCase());

        if (StringUtils.isEmpty(user.getPhonenumber())) {
            return ResponseVO.fail("手机号码不能为空！");
        } else if (!Validator.isMobile(user.getPhonenumber())) {
            return ResponseVO.fail("请输入正确的手机号码！");
        }
        // 加密手机号
        user.setPhonenumber(AesUtils.encryptHex(user.getPhonenumber()).toUpperCase());

        if (StringUtils.isEmpty(user.getNickname())) {
            return ResponseVO.fail("姓名不能为空！");
        } else if (user.getNickname().length() > 30) {
            return ResponseVO.fail("姓名长度不能超过30个字符！");
        }
        // 加密姓名
        user.setNickname(AesUtils.encryptHex(user.getNickname()).toUpperCase());

        try {
            if ("0".equals(user.getStatus())) {
                user.setErrNum(0);
                user.setLockTime(0L);
            }
            boolean bo = sysUserService.updateUser(user);
            if (bo) {
                return ResponseVO.success("修改用户信息成功");
            } else {
                log.error("修改用户信息失败");
                return ResponseVO.fail("修改用户信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("修改用户信息异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 初始密码发送邮箱
     */
    @PostMapping(value = "/resetPassword")
    @OperateLog(title = "用户管理", businessType = BusinessType.OTHER, auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:user:send')")
    public ResponseVO resetPassword(String id) {
        if (StringUtils.isEmpty(id)) {
            return ResponseVO.fail("重置密码缺失id!");
        }
        // id获取用户
        SysUser sysUser = sysUserService.getById(id);
        if (sysUser == null) {
            return ResponseVO.fail("重置密码失败！");
        }
        // 生成8位初始密码
        String randomPwd = RandomPwd.getRandomPwd(8);
        boolean bo = sysUserService.resetPassword(Integer.parseInt(id), SmUtil.sm3(randomPwd).toUpperCase());
        if (!bo) {
            return ResponseVO.fail("密码生成失败！");
        }
        try {
            sendMailUtil.executeSendMail(sysUser.getMailbox(), "系统登录密码", "密码：" + randomPwd);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseVO.fail("发送邮箱失败");
        }
        return ResponseVO.success();
    }

    /**
     * 删除用户信息
     */
    @PostMapping(value = "/delUser")
    @AgainVerify
    @OperateLog(title = "用户管理", businessType = BusinessType.DELETE, auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:user:remove')")
    public ResponseVO delete(String id) {
        if (StringUtils.isEmpty(id)) {
            return ResponseVO.fail("id不能为空!");
        }
        // id获取用户
        SysUser sysUser = sysUserService.getById(id);
        if (sysUser == null) {
            return ResponseVO.fail("不能删除用户！");
        }
        try {
            boolean bo = sysUserService.removeUserById(Integer.parseInt(id));
            if (bo) {
                return ResponseVO.success("删除用户信息成功");
            } else {
                log.error("删除用户信息失败");
                return ResponseVO.fail("删除用户信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除用户信息异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 修改密码
     */
    @PostMapping(value = "/updatePassword")
    @OperateLog(title = "用户管理", businessType = BusinessType.UPDATE, auditType = AuditType.SYS)
    public ResponseVO updatePassword(String id, String oldPassword, String newPassword, String confirmPassword) {
        if (StringUtils.isEmpty(id)) {
            return ResponseVO.fail("修改密码缺失id!");
        }
        // id获取用户
        SysUser sysUser = sysUserService.getById(id);
        if (sysUser == null) {
            return ResponseVO.fail("修改密码失败！");
        }

        if (StringUtils.isEmpty(oldPassword)) {
            return ResponseVO.fail("旧密码不能为空!");
        } else if (!sysUser.getPassword().equals(SmUtil.sm3(oldPassword).toUpperCase())) {
            return ResponseVO.fail("旧密码不正确！");
        }

        if (StringUtils.isEmpty(newPassword)) {
            return ResponseVO.fail("新密码不能为空!");
        } else if (StringUtils.isEmpty(confirmPassword)) {
            return ResponseVO.fail("确认密码不能为空！");
        } else if (!newPassword.equals(confirmPassword)) {
            return ResponseVO.fail("新密码两次输入的密码不一致！");
        }
        // 对新密码规则验证
        if (newPassword.contains(sysUser.getUsername())) {
            return ResponseVO.fail("密码不能含有账号！");
        }
        if (SmUtil.sm3(newPassword).toUpperCase().equals(sysUser.getPassword())) {
            return ResponseVO.fail("新密码不能与上次密码相同！");
        }
        if (RegexUtil.sameReg(newPassword)) {
            return ResponseVO.fail("新密码不能含有连续4位相同的数字或字母！");
        } else if (RegexUtil.keyboardSlopeArr(newPassword)) {
            return ResponseVO.fail("新密码不能含有4位斜方向连续的字符！");
        } else if (RegexUtil.keyboardHorizontalReg(newPassword)) {
            return ResponseVO.fail("新密码不能含有4位连续的字符！");
        } else if (!RegexUtil.checkPwd(newPassword)) {
            return ResponseVO.fail("新密码不满足8~20位大写字母、小写字母、数字、特殊字符三种以上的组合！");
        }

        sysUser.setPassword(SmUtil.sm3(newPassword).toUpperCase());
        sysUser.setLastUpdatePwdTime(new Date());
        boolean bo = sysUserService.updateById(sysUser);
        if (!bo) {
            return ResponseVO.fail("修改密码失败！");
        }
        return ResponseVO.success();
    }

    /**
     * 解锁用户信息
     */
    @PostMapping(value = "/relockUser")
    @OperateLog(title = "用户管理", businessType = BusinessType.OTHER, auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:user:relock')")
    public ResponseVO relockUser(String id) {
        if (StringUtils.isEmpty(id)) {
            return ResponseVO.fail("id不能为空!");
        }
        // id获取用户
        SysUser sysUser = sysUserService.getById(id);
        if (sysUser == null) {
            return ResponseVO.fail("不能解锁用户！");
        }
        if (!"1".equals(sysUser.getStatus())) {
            return ResponseVO.fail("只能对【锁定】状态的进行解锁！");
        }
        try {
            boolean bo = sysUserService.relockUserById(Integer.parseInt(id));
            if (bo) {
                return ResponseVO.success("解锁成功");
            } else {
                log.error("解锁失败");
                return ResponseVO.fail("解锁失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("解锁异常");
            return ResponseVO.error(e);
        }
    }
    /**
     * 用户授权角色
     */
    @PostMapping("/authRole")
    @PreAuthorize("@ss.hasPermi('system:user:role')")
    @OperateLog(title = "用户管理", businessType = BusinessType.GRANT, auditType = AuditType.SYS)
    public ResponseVO authRole(Long userId, Long roleId) {
        if (userId==null) {
            return ResponseVO.fail("用户id不能为空!");
        }
        try {
            sysUserService.insertUserAuth(userId, roleId);
            return ResponseVO.success("分配角色成功");
        } catch (Exception e) {
            e.printStackTrace();
            log.error("分配角色异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 根据用户ID获取角色
     * @param userId
     * @return
     */
    @GetMapping("/getUserRole")
    public ResponseVO getUserRole(Long userId) {
        if (userId==null) {
            return ResponseVO.fail("用户id不能为空!");
        }
        try {
            QueryWrapper<SysUserRole> wrapper = new QueryWrapper<>();
            wrapper.eq("user_id", userId);
            SysUserRole sysUserRole = sysUserRoleService.getOne(wrapper);
            return ResponseVO.success(sysUserRole);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("分配角色异常");
            return ResponseVO.error(e);
        }
    }
}
